package cz.vsb.fei.efrei.lab04;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.Set;

public class WorldOfCollections {
	private static final Random RANDOM = new Random();

	public static void main(String[] args) {
		useSetToStoreObjectWithoutHashCodeAndEquals();
		useSetToStoreObjectWithEqualsOnlyForName();
		useSetToStoreObjectWithEqualsAndHashCode();
		useSetToStoreObjectWithEqualsBasedOnNameAndBadHashCode();
		useSetToStoreObjectWithEqualsAndConstantHashCode();
		System.out.println("==========================================================================================");
		performanceForUseSetToStoreObjectWithEqualsAndHashCode();
		performanceForUseSetToStoreObjectWithEqualsAndConstantHashCode();
		System.out.println("==========================================================================================");
		performanceForAddIntoArrayList();
		performanceForAddIntoLinkedList();
		performanceForInsertIntoArrayList();
		performanceForInsertIntoLinkedList();
		
	}

	private static void testObjectAnHashSet(Set<Fighter> setOfFighters, Hero king1, Hero king2) {
		setOfFighters.add(king1);
		System.out.println("Is king1 " + king1 + " in set: " + setOfFighters.contains(king1));
		System.out.println("Is king1 " + king1 + " equal to king2 " + king2 + ": " + king1.equals(king2));
		System.out.println("Is king2 in set: " + setOfFighters.contains(king2));
		setOfFighters.add(king2);
		System.out.println("Is king2 in set after add: " + setOfFighters.contains(king2));
		System.out.println("Size of set is: " + setOfFighters.size());
	}

	public static void useSetToStoreObjectWithoutHashCodeAndEquals() {
		Set<Fighter> setOfFighters = new HashSet<>();
		Hero king1 = new Hero("King", 66);
		Hero king2 = new Hero("King", 66);
		System.out.println("=====Use HashSet to Store Object Without HashCode And Equals================================");
		testObjectAnHashSet(setOfFighters, king1, king2);
		System.out.println("Each object is different (different palces in memory) even look same!");
	}


	public static void useSetToStoreObjectWithEqualsOnlyForName() {
		Set<Fighter> setOfFighters = new HashSet<>();
		Hero king1 = new HeroWithEquals("King", 66);
		Hero king2 = new HeroWithEquals("King", 42);
		System.out.println("=====Use HashSet to Store Object Only With Equals================================");
		testObjectAnHashSet(setOfFighters, king1, king2);
		System.out.println("Object are equals because of same name, but not found in set because of worng hashcode!");
	}

	public static void useSetToStoreObjectWithEqualsAndHashCode() {
		Set<Fighter> setOfFighters = new HashSet<>();
		Hero king1 = new HeroWithEqualsAndHashcode("King", 66);
		Hero king2 = new HeroWithEqualsAndHashcode("King", 66);
		System.out.println("=====Use HashSet to Store Object Only With Equals================================");
		testObjectAnHashSet(setOfFighters, king1, king2);
		System.out.println("It's work!");
	}

	public static void useSetToStoreObjectWithEqualsBasedOnNameAndBadHashCode() {
		Set<Fighter> setOfFighters = new HashSet<>();
		Hero king1 = new HeroWithEqualsAndBadHashcode("King", 66);
		Hero king2 = new HeroWithEqualsAndBadHashcode("King", 42);
		System.out.println("=====Use HashSet to Store Object With Equals and BAD hashcode================================");
		testObjectAnHashSet(setOfFighters, king1, king2);
		System.out.println("It does'n work!");
		System.out.println("Object are equals because of same name, but not found in set because of worng hashcode calculated from strenght!");
	}

	public static void useSetToStoreObjectWithEqualsAndConstantHashCode() {
		Set<Fighter> setOfFighters = new HashSet<>();
		Hero king1 = new HeroWithEqualsAndConstantHashcode("King", 66);
		Hero king2 = new HeroWithEqualsAndConstantHashcode("King", 66);
		System.out.println("=====Use HashSet to Store Object With Equals and constant hashcode================================");
		testObjectAnHashSet(setOfFighters, king1, king2);
		System.out.println("It's work!");
	}
	
	private static String mesureAndAdd(Collection<Hero> sourceOfHeros, Collection<Hero> tragetOfHeros) {
		Stopwatch stopwatch = new Stopwatch();
		stopwatch.start();
		for (Hero hero : sourceOfHeros) {
			tragetOfHeros.add(hero);
		}
		stopwatch.stop();
		String duration = stopwatch.formatDuration();
		return duration;
	}

	private static String mesureAndInsert(Collection<Hero> sourceOfHeros, List<Hero> tragetOfHeros) {
		Stopwatch stopwatch = new Stopwatch();
		stopwatch.start();
		for (Hero hero : sourceOfHeros) {
			tragetOfHeros.add(0, hero);
		}
		stopwatch.stop();
		String duration = stopwatch.formatDuration();
		return duration;
	}

	public static void performanceForUseSetToStoreObjectWithEqualsAndHashCode() {
		int countOfHeros = 20000;
		List<Hero> heros = new ArrayList<>(countOfHeros);
		for (int i = 0; i < countOfHeros; i++) {
			heros.add(new HeroWithEqualsAndHashcode(Utils.generateHeroName()));
		}
		Set<Hero> setOfHeros = new HashSet<>();
		String duration = mesureAndAdd(heros, setOfHeros);
		System.out.println("Adding " + countOfHeros + " heros with correct hashcode and equals into set take " + duration);
	}


	public static void performanceForUseSetToStoreObjectWithEqualsAndConstantHashCode() {
		int countOfHeros = 20000;
		List<Hero> heros = new ArrayList<>(countOfHeros);
		for (int i = 0; i < countOfHeros; i++) {
			heros.add(new HeroWithEqualsAndConstantHashcode(Utils.generateHeroName()));
		}
		Set<Hero> setOfHeros = new HashSet<>();
		String duration = mesureAndAdd(heros, setOfHeros);
		System.out.println("Adding " + countOfHeros + " heros with constant hashcode into set take " + duration);
	}


	public static void performanceForAddIntoArrayList() {
		int countOfHeros = 500000;
		List<Hero> heros = new ArrayList<>(countOfHeros);
		for (int i = 0; i < countOfHeros; i++) {
			heros.add(new HeroWithEqualsAndConstantHashcode(Utils.generateHeroName()));
		}
		List<Hero> arrayListOfHeros = new ArrayList<>();
		String duration = mesureAndAdd(heros, arrayListOfHeros);
		System.out.println("Adding " + countOfHeros + " heros to end of ArrayList take " + duration);
	}

	public static void performanceForAddIntoLinkedList() {
		int countOfHeros = 500000;
		List<Hero> heros = new ArrayList<>(countOfHeros);
		for (int i = 0; i < countOfHeros; i++) {
			heros.add(new HeroWithEqualsAndConstantHashcode(Utils.generateHeroName()));
		}
		List<Hero> arrayListOfHeros = new LinkedList<>();
		String duration = mesureAndAdd(heros, arrayListOfHeros);
		System.out.println("Adding " + countOfHeros + " heros to end of LinkedList take " + duration);
	}

	public static void performanceForInsertIntoArrayList() {
		int countOfHeros = 500000;
		List<Hero> heros = new ArrayList<>(countOfHeros);
		for (int i = 0; i < countOfHeros; i++) {
			heros.add(new HeroWithEqualsAndConstantHashcode(Utils.generateHeroName()));
		}
		List<Hero> arrayListOfHeros = new ArrayList<>();
		String duration = mesureAndInsert(heros, arrayListOfHeros);
		System.out.println("Inserting " + countOfHeros + " heros at beginnig of ArrayList take " + duration);
	}

	public static void performanceForInsertIntoLinkedList() {
		int countOfHeros = 500000;
		List<Hero> heros = new ArrayList<>(countOfHeros);
		for (int i = 0; i < countOfHeros; i++) {
			heros.add(new HeroWithEqualsAndConstantHashcode(Utils.generateHeroName()));
		}
		List<Hero> arrayListOfHeros = new LinkedList<>();
		String duration = mesureAndInsert(heros, arrayListOfHeros);
		System.out.println("Inserting " + countOfHeros + " heros at beginnig of LinkedList take " + duration);
	}
}
